import axios from 'axios'
import { colors } from 'quasar'
const { hexToRgb } = colors

const $axios = axios.create({
  baseURL: 'https://www.web.statistik.zh.ch/zhwebcomponent/map/',
  withCredentials: false,
  headers: {
    'Access-Control-Allow-Origin' : '*'
  }
})
let availableKeys = null
const cachedMaps = []

export const zhutils = {
  dateToday () {
    return new Date().toISOString().split('T')[0]
  },
  sortToReadableDate (sortDate) {
    return sortDate.split('-').reverse().join('.')
  },
  getRandomInt (min, max) {
    min = Math.ceil(min)
    max = Math.floor(max)
    return Math.floor(Math.random() * (max - min + 1)) + min
  },
  pointAddition (p1, p2) {
    return { x: p1.x + p2.x, y: p1.y + p2.y }
  },
  pointOverlapRect (point, rect) {
    return zhutils.rectOverlapRect({ l: point.x, t: point.y, r: point.x + 0.00001, b: point.y + 0.00001 }, rect)
  },
  rectOverlapRect (r1, r2) {
    return Math.max(r1.l, r2.l) < Math.min(r1.r, r2.r) && Math.max(r1.t, r2.t) < Math.min(r1.b, r2.b)
  },
  rectCenter (rect) {
    return { x: (rect.l + rect.r) / 2, y: (rect.t + rect.b) / 2 }
  },
  rectSize (rect) {
    return { w: rect.r - rect.l, h: rect.b - rect.t }
  },
  rectInRectBounds (r1, r2) {
    const size = zhutils.rectSize(r1)
    return {
      l: Math.max(Math.min(r1.l, r2.r - size.w), r2.l),
      t: Math.max(Math.min(r1.t, r2.b - size.h), r2.t),
      r: Math.max(Math.min(r1.r, r2.r), r2.l + size.w),
      b: Math.max(Math.min(r1.b, r2.b), r2.t + size.h)
    }
  },
  combineRects (r1, r2) {
    return { l: r1.l + r2.l, t: r1.t + r2.t, r: r1.r + r2.r, b: r1.b + r2.b }
  },
  copyObj(obj) {
    return JSON.parse(JSON.stringify(obj))
  },
  isNumeric (numberString) {
    return !isNaN(parseFloat(numberString)) && isFinite(numberString)
  },
  formatNumber (number) {
    if (this.isNumeric(number)) {
      return number.toLocaleString('de-ch')
    }
    return number
  },
  valueBetween (start, end, amt) { // 0 - 1
    if (start === end) {
      return end
    }
    return parseFloat(((amt - start) * (1 / (end - start))).toFixed(3))
  },
  lerp (start, end, amt) {
    return (1 - amt) * start + amt * end
  },
  lerpColor (a, b, amount) { 
    const ah = parseInt(a.replace(/#/g, ''), 16),
      ar = ah >> 16, ag = ah >> 8 & 0xff, ab = ah & 0xff,
      bh = parseInt(b.replace(/#/g, ''), 16),
      br = bh >> 16, bg = bh >> 8 & 0xff, bb = bh & 0xff,
      rr = ar + amount * (br - ar),
      rg = ag + amount * (bg - ag),
      rb = ab + amount * (bb - ab)
    return '#' + ((1 << 24) + (rr << 16) + (rg << 8) + rb | 0).toString(16).slice(1)
  },
  contrastColor (hexString) {
    const rgb = hexToRgb(hexString)
    return ((rgb.r * 0.299) + (rgb.g * 0.587) + (rgb.b * 0.114)) > 130 ? '#000' : '#fff'
  },
  hexToRgb (hex) {
    const rgb = hex.replace(/^#?([a-f\d])([a-f\d])([a-f\d])$/i ,(m, r, g, b) => '#' + r + r + g + g + b + b)
      .substring(1).match(/.{2}/g)
      .map(x => parseInt(x, 16))
    return { r: rgb[0], g: rgb[1], b: rgb[2] }
  },
  rgbTohex({ r, g, b }) {
    r = Math.round(r)
    g = Math.round(g)
    b = Math.round(b)
    r = Math.min(r, 255)
    g = Math.min(g, 255)
    b = Math.min(b, 255)
    return '#' + [r, g, b].map(c => c.toString(16).padStart(2, '0')).join('')
  },
  colorOverlayOnWhite (hex, a) {
    const rgb = hexToRgb(hex)
    rgb.r = Math.floor(rgb.r * a + 0xff * (1 - a))
    rgb.g = Math.floor(rgb.g * a + 0xff * (1 - a))
    rgb.b = Math.floor(rgb.b * a + 0xff * (1 - a))
    return zhutils.rgbTohex(rgb)
  },
  arrayIntersectByKey (list1, list2, key = 'id') {
    const newArray = []
    list1.forEach(item1 => {
      const item2 = list2.find(o => o[key] === item1[key])
      if (item2) {
        newArray.push({ ...item2, ...item1 })
      }
    })
    return newArray
  },
  arraySum (array) {
    return array.reduce((a, b) => a + b, 0)
  },
  animate (start, end, duration, callback) {
    const props = {
      start: start,
      end: end,
      duration: duration,
      callback: callback,
      endTime: new Date().getTime() + duration // durtion in milisec
    }
    this.animateLoop(props)
  },
  animateLoop (props, now = new Date().getTime()) {
    const frames = 50
    const miliseconds = 1000 / frames
    if (now < props.endTime) {
      setTimeout(() => {
        props.callback(this.lerp(props.start, props.end, (props.duration - (props.endTime - now)) / props.duration))
        this.animateLoop(props)
      }, miliseconds)
    } else {
      props.callback(props.end)
    }
  },
  async getMapKeys () {
    return await new Promise(function(resolve) {
      if(availableKeys) {
        resolve(availableKeys)
      } else {
        $axios.get('summary.json').then(response => {
          availableKeys = response.data.groups.filter(o => o.filetype === 'json')
          resolve(availableKeys)
        })
      }
    })
  },
  async getMapDataWithKey (key, date) {
    return await new Promise(function(resolve) {
      const actualDate = date ? new Date(date).getTime() : new Date().getTime()
      if (cachedMaps[key + date]) {
        resolve(cachedMaps[key + date])
      } else {
        zhutils.getMapKeys().then(keys => {
          const k = keys.find(o => o.key === key)
          if (!k) {
            throw new Error(`map key ${key} not found in: [${ keys.map(o => o.key).join(', ') }]`)
          }
          const maps = k.maps
          const actualMap = maps.find(o => {
            return new Date(o.date_from).getTime() < actualDate && actualDate < new Date(o.date_to).getTime()
          })
          $axios.get(actualMap.url).then(response => {
            cachedMaps[key + date] = response.data
            resolve(cachedMaps[key + date])
          })
        })
      }
    })
  },
  svgObj (svg, x, y, fill, type) {
    const svgNS = 'http://www.w3.org/2000/svg'
    const obj = document.createElementNS(svgNS, type)
    obj.setAttributeNS(null, 'x', x)
    obj.setAttributeNS(null, 'y', y)
    obj.setAttributeNS(null, 'fill', fill)
    svg.appendChild(obj)
    return obj
  },
  svgRect (svg, x, y, width, height, fill) {
    const obj = zhutils.svgObj(svg, x, y, fill, 'rect')
    obj.setAttributeNS(null, 'width', width)
    obj.setAttributeNS(null, 'height', height)
    return obj
  },
  svgText (svg, text, textSize, x, y, fill) {
    const obj = zhutils.svgObj(svg, x, y, fill, 'text')
    obj.innerHTML = text
    obj.setAttributeNS(null, 'font-size', textSize)
    obj.setAttributeNS(null, 'dominant-baseline', 'hanging')
  },
  download (svg, title = 'SVG', filename = 'svg', type = 'svg') {
    const svgSource = svg.cloneNode(true)

    // zoom svg in to get space for Title and others
    const rect = svgSource.viewBox.baseVal
    const zoom = 1.2
    const h = rect.height
    rect.height = zoom * rect.height
    rect.y -= (rect.height - h) / 2

    const height = 500
    const width =  height / svg.getBoundingClientRect().height * svg.getBoundingClientRect().width
    svgSource.setAttribute('height', height)
    svgSource.setAttribute('width', width)
    svgSource.setAttribute('viewBox', `${rect.x} ${rect.y} ${rect.width} ${rect.height}`)
    svgSource.setAttribute('style', 'font-family:HelveticaNowBlack,"Arial Black",Helvetica,sans-serif')

    const svgNS = 'http://www.w3.org/2000/svg'
    const titleSize = rect.width / 20
    const quelleSize = rect.width / 50
    const margin = rect.width / 60

    // title bg
    zhutils.svgRect(svgSource, rect.x, rect.y + margin, rect.width, titleSize, '#FFFFFFCC')

    // title
    zhutils.svgText(svgSource, title, titleSize, rect.x, rect.y + margin, '#000')

    // quelle bg
    zhutils.svgRect(svgSource, rect.x, rect.y + rect.height - quelleSize - margin, rect.width, quelleSize, '#FFFFFFCC')

    // quelle
    const t = zhutils.sortToReadableDate(zhutils.dateToday()) + ' Quelle: Statistische Amt Kanton Zürich - ' + window.location.href
    zhutils.svgText(svgSource, t, quelleSize, rect.x, rect.y + rect.height - quelleSize - margin, '#999')

    const html = svgSource.outerHTML.replace(/<!---->/g, '')
    // download svg
    if (type === 'svg') {
      let imgURL = 'data:image/svg+xml;base64,' + btoa('<?xml version="1.0" encoding="ISO-8859-1" ?>' + html)
      zhutils.triggerDownload(imgURL, filename, 'svg')
    }
    if (type === 'png') {
      const blob = new Blob([html], { type: 'image/svg+xml;encoding=ISO-8859-1' })
      const URL = window.URL || window.webkitURL || window
      const imgURL = URL.createObjectURL(blob)
      let image = new Image()
      image.onload = () => {
        const canvas = document.createElement('canvas')
        canvas.width = width
        canvas.height = height
        const context = canvas.getContext('2d')
        context.drawImage(image, 0, 0, width, height)
        zhutils.triggerDownload(canvas.toDataURL(), filename, 'png')
      }
      image.src = imgURL
    }
  },
  triggerDownload (imgURL, filename, type) {
    const link = document.createElement('a')
    link.setAttribute('href', imgURL)
    link.setAttribute('download', filename + '.' + type)
    link.style.visibility = 'hidden'
    document.body.appendChild(link)
    link.click()
    document.body.removeChild(link)
  }
}